#!/usr/bin/python
import os
import sys
import struct
import time
import horde_api
import socket
from horde_ex import ReliableOrderedStream
from select import select


SESSION_NAME = "smart"
STREAM_NAME = "smart_stream"

hstream = 0
hconnect = 0



def session_cb(x):
    if (x == horde_api.EVENT_SESSION_CLOSED):
        print "session closed (%d). exiting." % x
        horde_api.session_close(SESSION_NAME)
        sys.exit(0)
    else:
        print "unknown session event:", x

def rate_to_kbps(rate):
    kbps = (rate * 8.0) / 1024;
    return "%.1f kbps" % kbps

def stat_cb(total_send, total_recv, total_allowed_send, nchannels):
    print "\tstatistics: send=%d (%s), recv=%d (%s), allowed=%d (%s), nchannels=%d"\
          % (total_send, rate_to_kbps(total_send), total_recv, rate_to_kbps(total_recv),
             total_allowed_send, rate_to_kbps(total_allowed_send), nchannels)

def horde_event_loop():
    while (horde_api.event_dispatch() > 0):
        pass

def close_cb(hstream):
    print "STREAM CLOSED:", hstream

def recv_cb(hstream, adu_id, data):
	global hconnect
	if hconnect:
		hconnect.send(data)
	print "RECV: stream=%d, adu=%d, data=\'%s\'" % (hstream, adu_id, data)
        
def do_client(server):
    global hstream, hconnect, tcpsock, addr
    
    if (horde_api.session_connect(server, SESSION_NAME)):
        print "connected to server '%s'." % server
        h = horde_api.stream_connect(STREAM_NAME)
        if (h > 0):
            hstream = h
            print "stream '%s' connected, uid:%d" % (STREAM_NAME, h)
            
            # register general callbacks
            horde_api.session_set_callback(session_cb)            
            horde_api.statistics_set_callback(stat_cb)
            horde_api.statistics_enable(0)
			
            # reliable stream
            
            host = "localhost"
            port = int(sys.argv[1])
            addr = (host,port)
            tcpsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            tcpsock.bind(addr)
            tcpsock.listen(5)
            hfd = horde_api.get_ippc_fd()
            rstream = ReliableOrderedStream(hstream, recv_cb, close_cb, True)
            adu_id = 0
            while True:
                hconnect, ipaq_addr = tcpsock.accept()
                

                # enter the send loop
                
                
                while True:
                    rlist,wlist,xlist = select([hconnect, hfd], [], [])
                    if hconnect in rlist:
                        data = hconnect.recv(1024)
                        if data == "":
                            break
                        rstream.send(adu_id, data, 0)
                        print "SENT\t%d\t%d\n" % (adu_id, len(data))
                        adu_id += 1
                    if hfd in rlist:
                        while horde_api.event_dispatch(0) > 0:
                            pass
                while horde_api.event_dispatch() > 0:
                            pass


def do_server():
    global hstream, hconnect
    
    if (horde_api.session_accept(SESSION_NAME)):
        print "client connected."
        h = horde_api.stream_accept(STREAM_NAME)
        if (h > 0):
            hstream = h
            print "stream '%s' connected, uid:%d" % (STREAM_NAME, h)
            
            # register callbacks
            horde_api.session_set_callback(session_cb)
            horde_api.statistics_set_callback(stat_cb)
            horde_api.statistics_enable(0)

            # reliable stream
            rstream = ReliableOrderedStream(hstream, recv_cb, close_cb, False)            
            
            host = "localhost"
            port = int(sys.argv[1])
            addr = (host,port)
            tcpsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            tcpsock.connect(addr)
            hconnect = tcpsock
            hfd = horde_api.get_ippc_fd()

            # enter the send loop
            adu_id = 0
            while True:
                rlist,wlist,xlist = select([hconnect, hfd], [], [])
                if hconnect in rlist:
                    data = hconnect.recv(1024)
                    if data == "":
                        break
                    rstream.send(adu_id, data, 0)
                    print "SENT\t%d\t%d\n" % (adu_id, len(data))
                    adu_id += 1
                if hfd in rlist:
                    while horde_api.event_dispatch(0) > 0:
                        pass
            while horde_api.event_dispatch() > 0:
                        pass

if len(sys.argv) == 2:
    # server running at Smart Central
    do_server()
elif len(sys.argv) == 3:
    # client connected to iPaq
    do_client(sys.argv[2])
else:
    print "usage:\n", "[As client]\t", sys.argv[0], "<tcp port> <horde server address>\n" \
			"[As server]\t", sys.argv[0], "<tcp port>"
