#ifndef __BLOCK_CODER_H__
#define __BLOCK_CODER_H__

//
// A simple block-based channel coder/decoder
//
#include <vector>
#include <map>
using namespace std;

class BlockCoder {
 private:
  size_t block_size;
  int window_size;
  vector<unsigned char*> window;
  unsigned char* coded;
 public:
  BlockCoder(size_t block_size, int window_size) : block_size(block_size), window_size(window_size) { 
    coded = new unsigned char[block_size];
  }
  ~BlockCoder() { delete [] coded; }

  /// returns non-null pointer to internal buffer if a coded block was produced at this boundary,
  /// also returns the new block's id, otherwise returns NULL.
  pair<unsigned char*,int> push(unsigned char* block, int blockid) {
    return push(block, blockid, block_size);
  }

  /// pushes when the valid bytes in block are size (which is < block_size)
  /// remaining bytes are treated as zeros.
  pair<unsigned char*,int> push(const unsigned char* block, int blockid, size_t size);
};

class BlockDecoder {
 private:
  void (*decoded_block_ready_cb)(unsigned char*, int, void*);
  void* cb_arg;
  size_t block_size;
  int window_size;

  typedef vector< pair<unsigned char*, bool> > window_t;
  typedef map<int, window_t > window_map_t;
  window_map_t windows;

  void _window_map_clear_row(window_t&);
  void _window_map_erase(int);
  void _window_map_limit_size();
 public:
  BlockDecoder(void (*decoded_block_ready_cb)(unsigned char*, int, void*), void* cb_arg, size_t block_size, int window_size) :
    decoded_block_ready_cb(decoded_block_ready_cb), block_size(block_size), window_size(window_size) { }

  /// calls the callback one or more times. if this is not a coded
  /// block, the callback is called exactly once. if this is a coded
  /// block, and enough information exists to reconstruct a normal
  /// block, the callback is called with the reconstructed block(s).
  void push(unsigned char* block, int blockid, size_t bsize);
};

#endif
