// -*- mode: C++; -*-

/** @file syscall.h
 *
 * Functions exported by the syscall module to the rest of the session
 * daemon.
 */
#ifndef __HORDE_SESSION_SYSCALL_H__
#define __HORDE_SESSION_SYSCALL_H__

#include <string>
#include "horde_namespace.h"

namespace nmstl {
  class io_event_loop; // no need to include nmstl/ioevent, just declare this class...
};

HORDE_NAMESPACE_BEGIN;

class ippc_handler;

/**
 * Principal in an operation on an I/O object (a stream, session, etc).
 * Only principals that 'own' objects can execute certain operations
 * on them.
 */
typedef ippc_handler* io_principal;

/**
 * Initialize the system call server. Starts an IPPC server bound to
 * the given path and with the given pipe being the IPPC socket that
 * owns the session.
 *
 * @return The handler created to manage the file descriptor \a ippc_owner_pipe_fd
 */
ippc_handler* init_syscall(nmstl::io_event_loop& ioloop, std::string path, int ippc_owner_pipe_fd);

HORDE_NAMESPACE_END;

#endif
