#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include <signal.h>

#include "ippc.h"
#include "syscall.h"

static volatile int quit = 0;

static void _sigint(int s) {
  printf("signal: %d\n", s);
  quit = 1;
}

void sleep(int timeout_ms) {
  timeval timeout;
  gettimeofday(&timeout, 0);
  timeout.tv_sec = (long)(timeout_ms) / 1000;;
  timeout.tv_usec = ((long)(timeout_ms) % 1000) * 1000;

  select(0, NULL, NULL, NULL, &timeout);
}

const size_t MSG_LEN = IPPC_MAX_MESSAGE_SIZE;

char sendbuf[MSG_LEN];

int nvals = (MSG_LEN - sizeof(ippcmsg_send_data)) / sizeof(int);

int main(void) {
  // setup signal handlers
  signal(SIGINT, _sigint);
  signal(SIGTERM, _sigint);
  signal(SIGHUP, _sigint);
  
  siginterrupt(SIGINT, 1);
  siginterrupt(SIGTERM, 1);
  siginterrupt(SIGHUP, 1);

  printf("sizeof ippcmsg_senddata: %d\n", sizeof(ippcmsg_send_data));
  
  // test code
  ippc_channel ioc = ippc_channel_open("/tmp/test.ippc");
  if (ioc == INVALID_CHANNEL) {
    perror("ippc_channel_open");    
  } else {
    int seq = 0;
    int rtype;
    size_t rlength;

    size_t txmsglen = sizeof(ippcmsg_send_data) + nvals * sizeof(int);
    ippcmsg_send_data* txmsg = (ippcmsg_send_data*)sendbuf;

    while (quit == 0) {
      sleep(1000);

      // prepare buffer
      int* iarr = (int*)(txmsg->data);
      iarr[0] = nvals;
      for (int i = 0; i < nvals-1; i++) {
	iarr[i+1] = seq+i;
      }
      txmsg->hstream = 0;
      txmsg->adu_id = seq;

      // send data
      printf("SEND %d: [%d, %d, %d, %d, ..., %d, %d, %d]\n", seq,
	     iarr[0], iarr[1], iarr[2], iarr[3], iarr[nvals-4], iarr[nvals-3], iarr[nvals-2]);
      char *msg = ippc_call(ioc, IPPC_SEND_DATA, txmsg, txmsglen, IPPC_RECV_DATA, &rtype, &rlength);
      
      if (msg == NULL) {
	perror("ippc_call");
	quit = 1;
      } else {
	if (rtype == IPPC_RECV_DATA) {
	  ippcmsg_recv_data* r = (ippcmsg_recv_data*)msg;
	  int* jarr = (int*)r->data;

	  int n = *jarr;
	  int miss=0;
	  for (int i = 0; i < n-1; i++) {
	    if (jarr[i+1] != r->adu_id+i) {
	      miss++;
	    }
	  }

	  printf("RECV_DATA: hstream=%d, adu_id=%d, frag_start=%d, miss=%d, [%d, %d, %d, ..., %d, %d]\n",
		 r->hstream, r->adu_id, r->frag_start, miss,
		 jarr[0], jarr[1], jarr[2], jarr[n-3], jarr[n-2]);
	} else {
	  // exception
	  ippcmsg_exception *ex = (ippcmsg_exception*)msg;
	  printf("Exception %d: '%s'\n", ex->code, ex->msg);
	}
      }
      seq++;
    }
  }
}
