// -*- mode: C++; -*-

/**
 * @file tunnel.h
 *
 * I/O handler to manage UDP sends and receives over tribe tunnels.
 * Also some constant definitions related to the tunnels.
 */

#ifndef __TRIBE_TUNNEL_H__
#define __TRIBE_TUNNEL_H__

#include <list>
#include <nmstl/netioevent>

using namespace std;
using namespace nmstl;

#define TUNNEL_MAX_PACKET_LEN 1500
#define TUNNEL_DEFAULT_PORT 9969

extern io_event_loop io_loop;

typedef void (*recv_handler_cb) (dynbuf&, inet_address&);

class tunnel_iohandler : public udp_iohandler {
private:
  recv_handler_cb recv_handler;
  inet_address peer_address;
public:
  tunnel_iohandler(inet_address bind_address, inet_address tunnel_leader_address, recv_handler_cb handle_recv) :
    udp_iohandler(io_loop, bind_address, true, false, false), peer_address(tunnel_leader_address), recv_handler(handle_recv)
  { 
    string peer;
    if (peer_address) peer = peer_address.as_string(); else peer = "*";

    INFO << "tunnel bound socket to: " << bind_address << " <-> " << peer;
  }

  void tunnel_send(dynbuf& buf, inet_address& sendto_address) {
    // assume socket is writable:
    int len = buf.offset_length();

    ssize_t n = send_packet(buf, sendto_address);
    
    if (n < len)
      WARN << "tunnel::send, some bytes dropped: written=" << n << ", total=" << len << ", errno: " << errno;
    else
      DEBUG(3) << "tunnel::send, len=" << len << ", dest: " << sendto_address.as_string();
  }

  void tunnel_send(dynbuf& buf) {
    tunnel_send(buf, peer_address);
  }

protected:
  void ravail() {
    dynbuf buf(TUNNEL_MAX_PACKET_LEN);
    inet_address addr;

    ssize_t n = recv_packet(buf, addr);
    DEBUG(3) << "tunnel::recv, n=" << n << ", len=" << buf.length() << ", from:" << addr;

    recv_handler(buf, addr);
  }
};

#endif
