// -*- mode: C++; -*-

/**
 * @file util.h
 *
 * Generic utility functions used by tribe components.
 */

#ifndef __TRIBE_UTIL_H__ 
#define __TRIBE_UTIL_H__

#include <string>
using namespace std;

//
// misc
//

/** Splits a string into a string array on space boundaries.
    The array is terminated with a NULL pointer.
    You'll need to call free_words on the returned array. */
char** split_words(string& src);
int count_words(char** warray); ///< counts num of elements before NULL.
void free_words(char** words);

/// parses lists of the form "a,b,c,d" into vectors
vector<string> parse_list(const char* liststr, char separator=',');

inline void tty_bold_on() { printf("\033[1;1m"); } ///< makes printf print in bold/bright
inline void tty_bold_off() { printf("\033[0m"); }  ///< stops printf from printing in bold/bright

//
// spawning processes
//

/**
 * Fork and then exec the given command. Optionally, redirect the stdout and stderr
 * of the child process to the given file descriptor.
 *
 * @return process id > 0 on success, or < 0 on error.
 */
int spawn(char* command, char *argv[], char *envp[], int stdout_redirect_fd=0);

int spawn(string& command);

//
// command line arguments
//

enum argument_type { 
  ARG_TYPE_NONE,
  ARG_TYPE_INTEGER,
  ARG_TYPE_BOOLEAN,
  ARG_TYPE_STRING
};

struct argument_table_element {
  argument_type type;
  char*         name;
  void*         var;
  bool          required;
  bool          parsed;
  char*         help;
};

extern argument_table_element END_OF_TABLE;

/**
 * Parses argv and writes any command line arguments to the right
 * place in the given argument table. The table must be terminated
 * by a END_OF_TABLE element.
 *
 * Returns false if not all required arguments were present in argv; or true otherwise.
 */
bool arguments_parse(char** argv, int nargv, argument_table_element args[]);

void arguments_print_usage(const char* progname, argument_table_element args[]);
void arguments_print_help(const char* progname, argument_table_element args[], bool log_level=false);

#endif
