#include <vector>
#include <sys/types.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "ifmanager.h"
#include "util.h"

#include <nmstl/debug>

using namespace std;
using namespace nmstl;

void(*exit_handler)() = NULL;
vector<int> child_pid;

string get_ifman_cmdline(string& iface) {
  char *cwd = get_current_dir_name();
  string cmd = cwd;

  cmd += "/ifmanager";

  cmd += " --iface=";
  cmd += iface;
  cmd += " --max-retry=";
  cmd += to_string(if_max_retry);
  cmd += " --retry-reset-period=";
  cmd += to_string(if_retry_reset_period);
  cmd += " --retry-sleep=";
  cmd += to_string(if_retry_sleep);
  if (if_reset_all) cmd += " --reset-all";

  return cmd;
}

void ifmanagers_start_all(vector<string>& managed_ifaces, void(*handler)()) {
  // set up event handler
  exit_handler = handler;

  // determine commandline
  
  // try to create and start all interface managers
  child_pid.clear();
  for (vector<string>::iterator i = managed_ifaces.begin(); i != managed_ifaces.end(); i++) {
    string com = get_ifman_cmdline(*i);
    int pid = spawn(com);

    if (pid > 0) {
      child_pid.push_back(pid);
    } else {
      // failure...
      ifmanagers_kill_all(); // kill all the active guys...
      exit_handler(); // notify exit handler
      return;
    }
  }
}

void ifmanagers_kill_all() {
  for (vector<int>::iterator i = child_pid.begin(); i != child_pid.end(); i++) {
    kill(*i, SIGHUP);
  }
  child_pid.clear();
}

void ifmanagers_poll() {
  if (waitpid(-1, NULL, WNOHANG) > 0) {
    if (exit_handler) exit_handler();
  }
}
